#version 130

// "uniform" variables which are user-determined
uniform int filter_width;
uniform int filter_inc;
uniform float zscale;
uniform float zbias;
uniform sampler2D texrgba;
uniform sampler2D texdepth;

// "out" variables are the outputs of this pipeline stage to the next
// this just outputs a single float which will be written to red
out vec4 fragment_colour;

// The main function runs once per "fragment" (per pixel)
void main () {
  vec2 viewsize, texcoord, xyoff, off;
  float depth, radius, weight, this_depth, this_radius, total_weight, obscure, depth_inc;
  vec4 colour, this_colour, total_colour;
  int x, y;

  // Always need texture coord for this fragment, and depth input
  viewsize = textureSize(texdepth, 0);
  texcoord = gl_FragCoord.xy / viewsize;
  depth = texture(texdepth, texcoord).r;
  radius = abs(zbias - depth) * zscale;
  colour = texture(texrgba, texcoord);

  // Ensure weighting is always non-zero
  total_weight = 0.00001;
  total_colour = total_weight * colour;
  obscure = 1.0 - 1.0 / max(1.0, radius);
  depth_inc = 0.001;
  xyoff = vec2(1.0, 1.0) / viewsize;

  // Calculate blur depending on disc size
  for (x=0; x<=(2*filter_width); x+=filter_inc) {
    for (y=0; y<=(2*filter_width); y+=filter_inc) {
      off = vec2(x-filter_width, y-filter_width);
      this_depth = texture(texdepth, texcoord + xyoff * off).r;
      this_radius = abs(zbias - this_depth) * zscale;
      this_colour = texture(texrgba, texcoord + xyoff * off);

      // Is the centre pixel within this pixel radius
      weight = smoothstep(0.0, 1.0, this_radius + 1.0 - length(off));

      // How spread out is the light within this disc
      weight *= 1.0 / max(1.0, this_radius * this_radius);

      // Pixels behind the focal point and the centre depth are partially obscured
      weight *= mix(1.0, obscure, smoothstep(0.0, depth_inc, this_depth - depth) * smoothstep(0.0, depth_inc, this_depth - zbias));
 
      // Want to pick out the high intensity details
      weight *= length(this_colour.rgb);

      total_weight += weight;
      total_colour += weight * this_colour;
    }
  }

  fragment_colour = total_colour / total_weight;
}
