#version 130

// "uniform" variables which are user-determined
uniform vec2 viewsize;
uniform int update_rgba;
uniform bool update_depth;
uniform sampler2D texrgba;
uniform sampler2D texdepth;

// "out" variables are the outputs of this pipeline stage to the next
// "fragment" shaders should output an RGBA pixel value
out vec4 fragment_colour;

// The main function runs once per "fragment" (per pixel)
void main () {
  vec2 texcoord = gl_FragCoord.xy / viewsize;

  // Possibly copy the texture to the output
  if (update_rgba == 2) {
    vec4 rgba = texture(texrgba, texcoord);
    fragment_colour = vec4(rgba.rgb, 1.0 - rgba.a);
  } else if (update_rgba == 1) {
    fragment_colour = texture(texrgba, texcoord);
  } else {
    fragment_colour = vec4(0);
  }

  // Possibly also copy depth to the output
  if (update_depth) {
    vec2 texsize = textureSize(texdepth, 0);
    vec2 texsteps = ceil(0.5 * ((texsize / viewsize) + 1)) - 1;
    if ((texsteps.x > 0) || (texsteps.y > 0)) {
      float x, y, d, d2;
      d = 1.0;
      vec2 texstep = 1.0 / texsize;
      for (y=-texsteps.y; y<=texsteps.y; y++) {
        for (x=-texsteps.x; x<=texsteps.x; x++) {
          d2 = texture(texdepth, texcoord + vec2(x, y)*texstep).r;
          d = min(d, d2);
        }
      }
      gl_FragDepth = d;
    } else {
      gl_FragDepth = texture(texdepth, texcoord).r;
    }
  } else {
    gl_FragDepth = gl_FragCoord.z;
  }
}
