#version 130

// "uniform" variables which are user-determined
uniform vec4 surface_colour;
uniform sampler2D texsurface;

// additional uniforms for if we are doing depth peeling front-to-back translucency
uniform bool depth_peeling;
uniform sampler2D texdepth0;
uniform sampler2D texdepth1;

// "in" variables are received from the previous pipeline stage
// "smooth" variables are bi-linearly interpolated between vertices
smooth in vec2 surface_texcoord;

// "out" variables are the outputs of this pipeline stage to the next
// "fragment" shaders should output an RGBA pixel value
out vec4 fragment_colour;

// The main function runs once per "fragment" (per pixel)
void main () {

  // Check additional front and back depth maps and possibly discard
  if (depth_peeling) {
    vec2 texcoord = gl_FragCoord.xy / textureSize(texdepth0, 0);
    // Allow for worst case floating point precision between 0.0 and 1.0
    float depth0 = texture(texdepth0, texcoord).r - 0.0000001;
    float depth1 = texture(texdepth1, texcoord).r + 0.0000001;
    if ((gl_FragCoord.z >= depth0) || (gl_FragCoord.z <= depth1)) {
      fragment_colour =  vec4(0.0, 0.0, 0.0, 1.0);
      gl_FragDepth = depth0 - 0.0000001;
      return;
    }
  }
  gl_FragDepth = gl_FragCoord.z;

  // Modulate the surface colour with the texture colour
  vec4 colour = surface_colour * texture(texsurface, surface_texcoord);
  if (colour.a == 0.0) discard;

  // may need to change alpha values for front-to-back translucency
  if (depth_peeling) {
    fragment_colour = vec4(colour.rgb*colour.a, 1.0 - colour.a);
  } else {
    fragment_colour = colour;
  }
}
