#version 130

// #define VARYING_COL to include per-pixel diffuse colour

// "uniform" variables which are user-determined
uniform mat4 modelview_matrix;
uniform mat4 modelviewproj_matrix;
uniform vec4 light1_position;
uniform bool radial_depth;

// "in" variables are supplied from the vertex data
in vec4 position;
in vec3 normal;
#ifdef VARYING_COL
in vec4 colour;
#endif // VARYING_COL

// "out" variables are the outputs of this pipeline stage to the next
// "vertex" shaders should output the built-in variable gl_Position,
// as well as anything declared here
smooth out vec3 N;
smooth out vec3 L1;
#ifdef VARYING_COL
smooth out vec4 surface_diffuse;
#endif // VARYING_COL

// The main function runs once per vertex
void main () {

#ifdef VARYING_COL
  // Copy the colour to the output
  surface_diffuse = colour;
#endif // VARYING_COL

  // Calculate the transformed normal vector for this vertex
  N = normalize(mat3(modelview_matrix) * normal);

  // Check for directional or point light 1
  if (light1_position.w != 0.0) {
    L1 = mat3(modelview_matrix) * (light1_position.xyz - position.xyz);
  } else {
    L1 = light1_position.xyz;
  }

  // Calculate the transformed and projected vertex position
  if (radial_depth) {
    vec4 ndc = modelviewproj_matrix * position;
    vec4 light1_ndc = modelviewproj_matrix * vec4(light1_position.xyz, 1.0);
    light1_ndc /= light1_ndc.w;
    float w, f;
    // get ndc and adjust for light location
    ndc.xyz /= ndc.w;
    ndc.xyz -= light1_ndc.xyz;
    if (light1_position.w < 0.0) ndc.z = -ndc.z;
    // project onto four planes which form a square-based pyramid from the light
    f = max(abs(ndc.x), abs(ndc.y));
    w = f + max(-f, ndc.z);
    w = max(w, 0.00001);
    // this changes z sign correctly so front-of-light is clipped
    // but need to use w (1/gl_Fragcoord.w) for depth in fragment shader
    ndc.z = (ndc.z - 1.0) * w;
    ndc.w = w;
    // scale x and y locations according to location of light
    ndc.xy += (w - f) * clamp(light1_ndc.xy, -1.0, 1.0);
    if (light1_position.w < 0.0) ndc.x = -ndc.x; // to preserve triangle winding
    ndc /= (max(abs(light1_ndc.x), abs(light1_ndc.y)) + abs(light1_ndc.z) + 2.0);
    gl_Position = ndc;
  } else {
    gl_Position = modelviewproj_matrix * position;
  }

}
