#version 130

// #define RADIANT_SHADOWS to include shadowing from light1 and light2 (straight on)
// #define OPAQUE_SHADOWS to also include shadowing on already drawn opaque surfaces

// "uniform" variables which are user-determined
uniform mat4 proj_matrix;
uniform vec4 light1_mv;
uniform vec3 light2_direction;
uniform vec3 gradient_x;
uniform vec3 gradient_y;
uniform vec3 gradient_z;
uniform vec3 pz;

// "in" variables are supplied from the vertex data
in vec4 position;
in vec3 texcoord;

// "out" variables are the outputs of this pipeline stage to the next
// "vertex" shaders should output the built-in variable gl_Position,
// as well as anything declared here
smooth out vec3 L1;
smooth out vec3 vol_texcoord;
smooth out vec3 vol_position_z;
smooth out vec2 light2_clip;
#ifdef RADIANT_SHADOWS
smooth out vec3 light1_clip;
smooth out vec3 light1_clip_start;
#endif // RADIANT_SHADOWS

// The main function runs once per vertex
void main () {

  // Calculate the transformed texture and projected vertex position
  vec4 pos = proj_matrix * position;
  gl_Position = pos;
  vol_texcoord = texcoord;

  // Check for directional or point light 1 (already in modelview coords)
  vec3 light1_direction;
  float L1len;
  if (light1_mv.w > 0.0) {
    light1_direction = light1_mv.xyz - position.xyz;
    L1 = light1_direction;
    L1len = length(light1_direction);
    light1_direction /= L1len;
  } else {
    light1_direction = light1_mv.xyz;
    L1 = light1_mv.xyz;
  }

  // Calculate the difference from one slice to the next in volume texture coordinates
  vec4 dz_h;
  vec3 light_start;
  if (abs(proj_matrix[3][3]) > 0.5) {
    // Orthographic
    vol_position_z = vec3(0.0, 0.0, 1.0);
    light_start = vec3(0.0, 0.0, -(proj_matrix[3][2] - 1.0)/proj_matrix[2][2]); // far plane
  } else {
    // Perspective - the following inverts the standard projection matrix for (pos - (0,0,1))
    dz_h.x = (pos.x + pos.w * proj_matrix[2][0]) / proj_matrix[0][0];
    dz_h.y = (pos.y + pos.w * proj_matrix[2][1]) / proj_matrix[1][1];
    dz_h.z = -pos.w;
    dz_h.w = (pos.z - pos.w + pos.w * proj_matrix[2][2]) / proj_matrix[3][2];
    vol_position_z = dz_h.xyz / dz_h.w - position.xyz / position.w;
    light_start = vec3(0.0, 0.0, -proj_matrix[3][2]/(proj_matrix[2][2] + 1.0)); // far plane
  }

  // Calculate the light movement for depth increment dz in texture coordinates
  float dz = pz.z;
  light2_clip = (proj_matrix * vec4(position.xyz - light2_direction * abs(dz / light2_direction.z), 1.0)).zw;

#ifdef RADIANT_SHADOWS
  vec3 lpos, pn = normalize(pz);
  float frame_count = dot(pn, light_start - position.xyz) / abs(dz * pn.z);
  lpos = light1_direction * abs(dz * pn.z) / max(abs(dot(light1_direction, pn)), 0.01);
  if (light1_mv.w > 0.0) {
    if (length(lpos) > L1len) lpos *= (L1len / length(lpos));
  }
  light1_clip_start = (proj_matrix * vec4(position.xyz + lpos * frame_count, 1.0)).xyw;
  light1_clip = (proj_matrix * vec4(position.xyz + lpos, 1.0)).xyw;
#endif // RADIANT_SHADOWS

}
